//! # Notifications controller
//!
//! @author Pierre Hubert

use crate::api_data::notification_api::NotificationAPI;
use crate::api_data::res_count_all_unreads::ResCountAllUnread;
use crate::api_data::res_number_unread_notifications::ResNumberUnreadNotifications;
use crate::controllers::routes::RequestResult;
use crate::data::error::ResultBoxError;
use crate::data::http_request_handler::HttpRequestHandler;
use crate::data::notification::PartialNotification;
use crate::helpers::{conversations_helper, friends_helper, notifications_helper};

impl HttpRequestHandler {
    /// Get the id of a notification included in the request
    pub fn post_notif_id(&mut self, name: &str) -> ResultBoxError<u64> {
        let notif_id = self.post_u64(name)?;

        let notif = PartialNotification::new()
            .set_id(notif_id)
            .set_dest_user_id(self.user_id_ref()?);

        if !notifications_helper::similar_exists(&notif)? {
            self.not_found("Specified notification not found!".to_string())?;
        }

        Ok(notif_id)
    }
}

/// Count the number of unread notifications
pub fn count_unread(r: &mut HttpRequestHandler) -> RequestResult {
    let number = notifications_helper::count_unread(r.user_id_ref()?)?;
    r.set_response(ResNumberUnreadNotifications::new(number))
}

/// Count the number of unread notifications
pub fn count_all_news(r: &mut HttpRequestHandler) -> RequestResult {
    let notifications = notifications_helper::count_unread(r.user_id_ref()?)?;
    let conversations = conversations_helper::count_unread_for_user(r.user_id_ref()?)?;
    let friends_requests = match r.post_bool_opt("friends_request", false) {
        true => Some(friends_helper::count_requests(r.user_id_ref()?)?),
        false => None
    };

    r.set_response(ResCountAllUnread::new(notifications, conversations as u64, friends_requests))
}

/// Get the list of unread notifications
pub fn get_list_unread(r: &mut HttpRequestHandler) -> RequestResult {
    let list = notifications_helper::get_list_unread(r.user_id_ref()?)?;

    r.set_response(NotificationAPI::for_list(&list))
}

/// Mark a notification as seen
pub fn mark_seen(r: &mut HttpRequestHandler) -> RequestResult {
    let notif_id = r.post_notif_id("notifID")?;
    let delete_similar = r.post_bool_opt("delete_similar", false);

    // Check if we are targeting a precise notification or an undetermined number of similar
    // notifications
    if !delete_similar {
        notifications_helper::delete(
            &PartialNotification::new()
            .set_id(notif_id)
        )?;
    }

    else {
        // TODO : implement me
        unimplemented!();
    }

    r.success("Notification deleted")
}